@@ -1,7 +1,7 @@
 /* addr_resolv.c
  * Routines for network object lookup
  *
- * $Id: addr_resolv.c 27718 2009-03-13 22:06:48Z stig $
+ * $Id: addr_resolv.c 27920 2009-04-01 00:30:45Z gerald $
  *
  * Laurent Deniel <laurent.deniel@free.fr>
  *
@@ -115,7 +115,7 @@
 # ifdef HAVE_GNU_ADNS
 #  include <errno.h>
 #  include <adns.h>
-#  ifdef inet_aton
+#  if defined(inet_aton) && defined(_WIN32)
 #   undef inet_aton
 #  endif
 # endif	/* HAVE_GNU_ADNS */
