@@ -5,7 +5,7 @@
  *
  * Copyright 2001, 2004 Michael Tuexen <tuexen [AT] fh-muenster.de>
  *
- * $Id: packet-mtp2.c 23278 2007-10-26 16:19:18Z morriss $
+ * $Id: packet-mtp2.c 27890 2009-03-29 22:16:26Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -56,13 +56,12 @@ static int hf_mtp2_ext_li    = -1;
 static int hf_mtp2_spare     = -1;
 static int hf_mtp2_ext_spare = -1;
 static int hf_mtp2_sf        = -1;
-static int hf_mtp2_long_sf   = -1;
+static int hf_mtp2_sf_extra  = -1;
 
 /* Initialize the subtree pointers */
 static gint ett_mtp2       = -1;
 
 static dissector_handle_t mtp3_handle;
-static int mtp3_proto_id;
 static gboolean use_extended_sequence_numbers_default = FALSE;
 static gboolean use_extended_sequence_numbers         = FALSE;
 
@@ -129,7 +128,8 @@ dissect_mtp2_header(tvbuff_t *su_tvb, proto_item *mtp2_tree)
 * DETAILS : Calculate a new FCS-16 given the current FCS-16 and the new data.
 *******************************************************************************
 */
-static guint16 mtp2_fcs16(tvbuff_t * tvbuff)
+static guint16
+mtp2_fcs16(tvbuff_t * tvbuff)
 {
     guint len = tvb_length(tvbuff)-2;
 
@@ -142,7 +142,8 @@ static guint16 mtp2_fcs16(tvbuff_t * tvbuff)
 /*
  * This function for CRC16 only is based on the decode_fcs of packet_ppp.c
  */
-tvbuff_t * mtp2_decode_crc16(tvbuff_t *tvb, proto_tree *fh_tree, packet_info *pinfo)
+static tvbuff_t *
+mtp2_decode_crc16(tvbuff_t *tvb, proto_tree *fh_tree, packet_info *pinfo)
 {
   tvbuff_t   *next_tvb;
   gint       len, reported_len;
@@ -239,28 +240,35 @@ static const value_string status_field_acro_vals[] = {
 #define SF_OFFSET          (LI_OFFSET + LI_LENGTH)
 #define EXTENDED_SF_OFFSET (EXTENDED_LI_OFFSET + EXTENDED_LI_LENGTH)
 
-#define SF_LENGTH          1
-#define LONG_SF_LENGTH     2
+#define SF_LENGTH			1
+#define SF_EXTRA_OFFSET			(SF_OFFSET + SF_LENGTH)
+#define EXTENDED_SF_EXTRA_OFFSET	(EXTENDED_SF_OFFSET + SF_LENGTH)
+#define SF_EXTRA_LENGTH			1
 
 static void
 dissect_mtp2_lssu(tvbuff_t *su_tvb, packet_info *pinfo, proto_item *mtp2_tree)
 {
   guint8 sf = 0xFF;
+  guint8 sf_offset, sf_extra_offset;
   
   if (use_extended_sequence_numbers) {
-    if ((tvb_get_letohs(su_tvb, EXTENDED_LI_OFFSET) & EXTENDED_LI_MASK) == 1) {
-      proto_tree_add_item(mtp2_tree, hf_mtp2_sf,      su_tvb, EXTENDED_SF_OFFSET, SF_LENGTH,      LITTLE_ENDIAN_BYTE_ORDER);
-      sf = tvb_get_guint8(su_tvb, EXTENDED_SF_OFFSET);
-    } else
-      proto_tree_add_item(mtp2_tree, hf_mtp2_long_sf, su_tvb, EXTENDED_SF_OFFSET, LONG_SF_LENGTH, LITTLE_ENDIAN_BYTE_ORDER);
+    sf_offset = EXTENDED_SF_OFFSET;
+    sf_extra_offset = EXTENDED_SF_EXTRA_OFFSET;
   } else {
-    if ((tvb_get_guint8(su_tvb, LI_OFFSET) & LI_MASK) == 1) {
-      proto_tree_add_item(mtp2_tree, hf_mtp2_sf,      su_tvb, SF_OFFSET,          SF_LENGTH,      LITTLE_ENDIAN_BYTE_ORDER);
-      sf = tvb_get_guint8(su_tvb, SF_OFFSET);
-    } else
-      proto_tree_add_item(mtp2_tree, hf_mtp2_long_sf, su_tvb, SF_OFFSET,          LONG_SF_LENGTH, LITTLE_ENDIAN_BYTE_ORDER);
+    sf_offset = SF_OFFSET;
+    sf_extra_offset = SF_EXTRA_OFFSET;
   }
 
+  proto_tree_add_item(mtp2_tree, hf_mtp2_sf, su_tvb, sf_offset, SF_LENGTH, LITTLE_ENDIAN_BYTE_ORDER);
+  sf = tvb_get_guint8(su_tvb, SF_OFFSET);
+
+  /*  If the LI is 2 then there is an extra octet following the standard SF
+   *  field but it is not defined what this octet is.
+   *  (In any case the first byte of the SF always has the same meaning.)
+   */
+  if ((tvb_get_guint8(su_tvb, LI_OFFSET) & LI_MASK) == 2)
+    proto_tree_add_item(mtp2_tree, hf_mtp2_sf_extra, su_tvb, sf_extra_offset, SF_EXTRA_LENGTH, LITTLE_ENDIAN_BYTE_ORDER);
+
   if (check_col(pinfo->cinfo, COL_INFO))
     col_set_str(pinfo->cinfo, COL_INFO, val_to_str(sf, status_field_acro_vals, "Unknown"));
 }
@@ -367,21 +375,21 @@ proto_register_mtp2(void)
 {
 
   static hf_register_info hf[] = {
-    { &hf_mtp2_bsn,       { "Backward sequence number", "mtp2.bsn",   FT_UINT8,  BASE_DEC, NULL,                    BSN_MASK,            "", HFILL } },
-    { &hf_mtp2_ext_bsn,   { "Backward sequence number", "mtp2.bsn",   FT_UINT16, BASE_DEC, NULL,                    EXTENDED_BSN_MASK,   "", HFILL } },
-    { &hf_mtp2_ext_res,   { "Reserved",                 "mtp2.res",   FT_UINT16, BASE_DEC, NULL,                    EXTENDED_RES_MASK,   "", HFILL } },
-    { &hf_mtp2_bib,       { "Backward indicator bit",   "mtp2.bib",   FT_UINT8,  BASE_DEC, NULL,                    BIB_MASK,            "", HFILL } },
-    { &hf_mtp2_ext_bib,   { "Backward indicator bit",   "mtp2.bib",   FT_UINT16, BASE_DEC, NULL,                    EXTENDED_BIB_MASK,   "", HFILL } },
-    { &hf_mtp2_fsn,       { "Forward sequence number",  "mtp2.fsn",   FT_UINT8,  BASE_DEC, NULL,                    FSN_MASK,            "", HFILL } },
-    { &hf_mtp2_ext_fsn,   { "Forward sequence number",  "mtp2.fsn",   FT_UINT16, BASE_DEC, NULL,                    EXTENDED_FSN_MASK,   "", HFILL } },
-    { &hf_mtp2_fib,       { "Forward indicator bit",    "mtp2.fib",   FT_UINT8,  BASE_DEC, NULL,                    FIB_MASK,            "", HFILL } },
-    { &hf_mtp2_ext_fib,   { "Forward indicator bit",    "mtp2.fib",   FT_UINT16, BASE_DEC, NULL,                    EXTENDED_FIB_MASK,   "", HFILL } },
-    { &hf_mtp2_li,        { "Length Indicator",         "mtp2.li",    FT_UINT8,  BASE_DEC, NULL,                    LI_MASK,             "", HFILL } },
-    { &hf_mtp2_ext_li,    { "Length Indicator",         "mtp2.li",    FT_UINT16, BASE_DEC, NULL,                    EXTENDED_LI_MASK,    "", HFILL } },
-    { &hf_mtp2_spare,     { "Spare",                    "mtp2.spare", FT_UINT8,  BASE_DEC, NULL,                    SPARE_MASK,          "", HFILL } },
-    { &hf_mtp2_ext_spare, { "Spare",                    "mtp2.spare", FT_UINT16, BASE_DEC, NULL,                    EXTENDED_SPARE_MASK, "", HFILL } },
-    { &hf_mtp2_sf,        { "Status field",             "mtp2.sf",    FT_UINT8,  BASE_DEC, VALS(status_field_vals), 0x0,                 "", HFILL } },
-    { &hf_mtp2_long_sf,   { "Status field",             "mtp2.sf",    FT_UINT16, BASE_HEX, NULL,                    0x0,                 "", HFILL } }
+    { &hf_mtp2_bsn,       { "Backward sequence number", "mtp2.bsn",      FT_UINT8,  BASE_DEC, NULL,                    BSN_MASK,            NULL, HFILL } },
+    { &hf_mtp2_ext_bsn,   { "Backward sequence number", "mtp2.bsn",      FT_UINT16, BASE_DEC, NULL,                    EXTENDED_BSN_MASK,   NULL, HFILL } },
+    { &hf_mtp2_ext_res,   { "Reserved",                 "mtp2.res",      FT_UINT16, BASE_DEC, NULL,                    EXTENDED_RES_MASK,   NULL, HFILL } },
+    { &hf_mtp2_bib,       { "Backward indicator bit",   "mtp2.bib",      FT_UINT8,  BASE_DEC, NULL,                    BIB_MASK,            NULL, HFILL } },
+    { &hf_mtp2_ext_bib,   { "Backward indicator bit",   "mtp2.bib",      FT_UINT16, BASE_DEC, NULL,                    EXTENDED_BIB_MASK,   NULL, HFILL } },
+    { &hf_mtp2_fsn,       { "Forward sequence number",  "mtp2.fsn",      FT_UINT8,  BASE_DEC, NULL,                    FSN_MASK,            NULL, HFILL } },
+    { &hf_mtp2_ext_fsn,   { "Forward sequence number",  "mtp2.fsn",      FT_UINT16, BASE_DEC, NULL,                    EXTENDED_FSN_MASK,   NULL, HFILL } },
+    { &hf_mtp2_fib,       { "Forward indicator bit",    "mtp2.fib",      FT_UINT8,  BASE_DEC, NULL,                    FIB_MASK,            NULL, HFILL } },
+    { &hf_mtp2_ext_fib,   { "Forward indicator bit",    "mtp2.fib",      FT_UINT16, BASE_DEC, NULL,                    EXTENDED_FIB_MASK,   NULL, HFILL } },
+    { &hf_mtp2_li,        { "Length Indicator",         "mtp2.li",       FT_UINT8,  BASE_DEC, NULL,                    LI_MASK,             NULL, HFILL } },
+    { &hf_mtp2_ext_li,    { "Length Indicator",         "mtp2.li",       FT_UINT16, BASE_DEC, NULL,                    EXTENDED_LI_MASK,    NULL, HFILL } },
+    { &hf_mtp2_spare,     { "Spare",                    "mtp2.spare",    FT_UINT8,  BASE_DEC, NULL,                    SPARE_MASK,          NULL, HFILL } },
+    { &hf_mtp2_ext_spare, { "Spare",                    "mtp2.spare",    FT_UINT16, BASE_DEC, NULL,                    EXTENDED_SPARE_MASK, NULL, HFILL } },
+    { &hf_mtp2_sf,        { "Status field",             "mtp2.sf",       FT_UINT8,  BASE_DEC, VALS(status_field_vals), 0x0,                 NULL, HFILL } },
+    { &hf_mtp2_sf_extra,  { "Status field extra octet", "mtp2.sf_extra", FT_UINT8,  BASE_HEX, NULL,                    0x0,                 NULL, HFILL } }
   };
 
   static gint *ett[] = {
@@ -411,11 +419,9 @@ proto_reg_handoff_mtp2(void)
 {
   dissector_handle_t mtp2_handle;
 
-  mtp2_handle = create_dissector_handle(dissect_mtp2, proto_mtp2);
-
+  mtp2_handle = find_dissector("mtp2");
   dissector_add("wtap_encap", WTAP_ENCAP_MTP2, mtp2_handle);
   dissector_add("wtap_encap", WTAP_ENCAP_MTP2_WITH_PHDR, mtp2_handle);
 
   mtp3_handle   = find_dissector("mtp3");
-  mtp3_proto_id = proto_get_id_by_filter_name("mtp3");
 }
