@@ -7,7 +7,7 @@ proper helper routines
  * Routines for dissection of ASN.1 Aligned PER
  * 2003  Ronnie Sahlberg
  *
- * $Id: packet-per.c 25839 2008-07-26 15:28:04Z kukosa $
+ * $Id: packet-per.c 28374 2009-05-15 18:39:33Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -75,6 +75,7 @@ static int hf_per_encoding = -1;                  /* External_encoding */
 static int hf_per_single_ASN1_type = -1;          /* T_single_ASN1_type */
 static int hf_per_octet_aligned = -1;             /* T_octet_aligned */
 static int hf_per_arbitrary = -1;                 /* T_arbitrary */
+static int hf_per_integer_length = -1;			  /* Show integer length if "show internal per fields" */
 
 static gint ett_per_open_type = -1;
 static gint ett_per_containing = -1;
@@ -140,8 +141,7 @@ static tvbuff_t *new_octet_aligned_subset(tvbuff_t *tvb, guint32 offset, asn1_ct
       octet0 = tvb_get_guint8(tvb, boffset + i + 1);
       buf[i] = (octet1 << shift1) | (octet0 >> shift0);
     }
-    sub_tvb = tvb_new_real_data(buf, actual_length, length);
-    tvb_set_child_real_data_tvbuff(tvb, sub_tvb);
+    sub_tvb = tvb_new_child_real_data(tvb, buf, actual_length, length);
     add_new_data_source(actx->pinfo, sub_tvb, "Unaligned OCTET STRING");
   } else {  /* aligned */
     sub_tvb = tvb_new_subset(tvb, boffset, actual_length, length);
@@ -161,9 +161,9 @@ static const guint16 bit_mask16[] = {
 };
 
 /* Fetch a number of bits to a new tvb right adjusted to the nearest number of bytes.
- * (add proceeding zeros).
+ * (add proceeding zeros in case of aligned PER)
  */
-static tvbuff_t *new_octet_aligned_subset_bits(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, guint32 no_of_bits)
+tvbuff_t *new_octet_aligned_subset_bits(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, guint32 no_of_bits)
 {
   tvbuff_t *sub_tvb = NULL;
   guint32 boffset = offset >> 3;
@@ -186,26 +186,49 @@ static tvbuff_t *new_octet_aligned_subset_bits(tvbuff_t *tvb, guint32 offset, as
   }
   buf = ep_alloc(length);
 
-  /* get the 'odd' bits */
-  shift1 = offset & 0x07;
-  word = tvb_get_ntohs(tvb,boffset) & bit_mask16[offset & 0x07];
-  word = word >> (16-(shift1+remainder));
-  buf[0] = word & 0x00ff;
-
-  offset = offset + remainder;
-  boffset = offset >> 3;
-  if (length >1){
+  if (actx->aligned)
+  {
+    /* get the 'odd' bits */
     shift1 = offset & 0x07;
-    shift0 = 8 - shift1;
-    octet0 = tvb_get_guint8(tvb, boffset);
-    for (i=1; i<length; i++) {
-      octet1 = octet0;
-      octet0 = tvb_get_guint8(tvb, boffset + i);
-      buf[i] = (octet1 << shift1) | (octet0 >> shift0);
+    word = tvb_get_ntohs(tvb,boffset) & bit_mask16[offset & 0x07];
+    word = word >> (16-(shift1+remainder));
+    buf[0] = word & 0x00ff;
+
+    offset = offset + remainder;
+    boffset = offset >> 3;
+    if (length >1){
+      shift1 = offset & 0x07;
+      shift0 = 8 - shift1;
+      octet0 = tvb_get_guint8(tvb, boffset);
+      for (i=1; i<length; i++) {
+        octet1 = octet0;
+        octet0 = tvb_get_guint8(tvb, boffset + i);
+        buf[i] = (octet1 << shift1) | (octet0 >> shift0);
+      }
     }
   }
-  sub_tvb = tvb_new_real_data(buf, length, length);
-  tvb_set_child_real_data_tvbuff(tvb, sub_tvb);
+  else
+  {
+    /* Do not preceed with zeros in case of PER unaligned */
+    i = 0;
+    shift1 = offset & 0x07;
+    shift0 = 8 - shift1;
+
+    if (length > 1){
+      octet0 = tvb_get_guint8(tvb, boffset);
+      for (; i < length-1; i++) {
+        octet1 = octet0;
+        octet0 = tvb_get_guint8(tvb, boffset + i + 1);
+        buf[i] = (octet1 << shift1) | (octet0 >> shift0);
+      }
+    }
+    /* get the 'odd' bits */
+    word = tvb_get_ntohs(tvb,boffset+i) << shift1;
+    word = word & ~bit_mask16[remainder];
+    word = word >> 8;
+    buf[i] = (guint8) (word & 0x00ff);
+  }
+  sub_tvb = tvb_new_child_real_data(tvb, buf, length, length);
   add_new_data_source(actx->pinfo, sub_tvb, "Unaligned OCTET STRING");
   
   return sub_tvb;
@@ -360,11 +383,18 @@ dissect_per_length_determinant(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx _
 			if(tmp){
 				val|=1;
 				g_strlcat(str, "1", 256);
+				if (i==0) { /* bit 8 is 1, so not a single byte length */
+					num_bits = 16;
+				}
+				else if (i==1 && val==3) { /* bits 8 and 7 both 1, so unconstrained */
+					PER_NOT_DECODED_YET("10.9 Unconstrained");
+					return offset;
+				}
 			} else {
 				g_strlcat(str, "0", 256);
 			}
 		}
-		if((val&0x80)==0){
+		if((val&0x80)==0 && num_bits==8){
 			*length = val;
 			if(hf_index!=-1){
 				pi = proto_tree_add_uint(tree, hf_index, tvb, (offset>>3)-1, 1, *length);
@@ -376,6 +406,18 @@ dissect_per_length_determinant(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx _
 		
 			return offset;
 		}
+		else if (num_bits==16) {
+			*length = val&0x3fff;
+			if(hf_index!=-1){
+				pi = proto_tree_add_uint(tree, hf_index, tvb, (offset>>3)-1, 1, *length);
+				if (display_internal_per_fields)
+					proto_item_append_text(pi," %s", str);
+				else
+					PROTO_ITEM_SET_HIDDEN(pi);
+			}
+        
+			return offset;
+		}
 		PER_NOT_DECODED_YET("10.9 Unaligned");
 		return offset;
 
@@ -410,7 +452,7 @@ dissect_per_length_determinant(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx _
 static guint32
 dissect_per_normally_small_nonnegative_whole_number(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *tree, int hf_index, guint32 *length)
 {
-	gboolean small_number;
+	gboolean small_number, length_bit;
 	guint32 len;
 	proto_item *pi;
 
@@ -426,18 +468,14 @@ DEBUG_ENTRY("dissect_per_normally_small_nonnegative_whole_number");
 		/* 10.6.1 */
 		*length=0;
 		for(i=0;i<6;i++){
-			offset=dissect_per_boolean(tvb, offset, actx, tree, -1, &small_number);
+			offset=dissect_per_boolean(tvb, offset, actx, tree, -1, &length_bit);
 			*length<<=1;
-			if(small_number){
+			if (length_bit) {
 				*length|=1;
 			}
 		}
 		if(hf_index!=-1){
-			if((offset&0x07)<7){
-				pi = proto_tree_add_uint(tree, hf_index, tvb, (offset>>3)-1, 1, *length);
-			} else {
-				pi = proto_tree_add_uint(tree, hf_index, tvb, (offset>>3), 1, *length);
-			}
+			pi = proto_tree_add_uint(tree, hf_index, tvb, (offset-6)>>3, (offset%8<6)?2:1, *length);
 			if (!display_internal_per_fields) PROTO_ITEM_SET_HIDDEN(pi);
 		}
 		return offset;
@@ -478,7 +516,7 @@ guint32
 dissect_per_null(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index) {
   proto_item *ti_tmp;
 
-  ti_tmp = proto_tree_add_item(tree, hf_index, tvb, offset>>8, 0, FALSE);
+  ti_tmp = proto_tree_add_item(tree, hf_index, tvb, offset>>3, 1, FALSE);
   proto_item_append_text(ti_tmp, ": NULL");
 
   return offset;
@@ -565,8 +603,7 @@ DEBUG_ENTRY("dissect_per_restricted_character_string");
 	/* xx.x if the length is 0 bytes there will be no encoding */
 	if(max_len==0){
 		if (value_tvb) {
-			*value_tvb = tvb_new_real_data(NULL, 0, 0); 
-			tvb_set_child_real_data_tvbuff(tvb, *value_tvb);
+			*value_tvb = tvb_new_child_real_data(tvb, NULL, 0, 0); 
 		}
 		return offset;
 	}
@@ -671,9 +708,8 @@ DEBUG_ENTRY("dissect_per_restricted_character_string");
 	buf[char_pos]=0;
 	proto_tree_add_string(tree, hf_index, tvb, (old_offset>>3), (offset>>3)-(old_offset>>3), (char*)buf);
 	if (value_tvb) {
-		*value_tvb = tvb_new_real_data(buf, length, length); 
+		*value_tvb = tvb_new_child_real_data(tvb, buf, length, length); 
 		tvb_set_free_cb(*value_tvb, g_free);
-		tvb_set_child_real_data_tvbuff(tvb, *value_tvb);
 	} else {
 		g_free(buf);
 	}
@@ -793,7 +829,7 @@ dissect_per_object_descriptor(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, p
 
 /* this function dissects a constrained sequence of */
 guint32
-dissect_per_constrained_sequence_of(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *parent_tree, int hf_index, gint ett_index, const per_sequence_t *seq, int min_len, int max_len)
+dissect_per_constrained_sequence_of(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *parent_tree, int hf_index, gint ett_index, const per_sequence_t *seq, int min_len, int max_len, gboolean has_extension _U_)
 {
 	proto_item *item;
 	proto_tree *tree;
@@ -803,6 +839,22 @@ dissect_per_constrained_sequence_of(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *a
 
 DEBUG_ENTRY("dissect_per_constrained_sequence_of");
 
+	/* 19.4	If there is a PER-visible constraint and an extension marker is present in it, 
+	 * a single bit shall be added to the field-list in a bit-field of length one
+	 */
+	if(has_extension){
+		gboolean extension_present;
+		offset=dissect_per_boolean(tvb, offset, actx, parent_tree, hf_per_extension_present_bit, &extension_present);
+		if (!display_internal_per_fields) PROTO_ITEM_SET_HIDDEN(actx->created_item);
+		if(extension_present){
+			/* 10.9 shall be invoked to add the length determinant as a semi-constrained whole number to the field-list, 
+			 * followed by the component values
+			 * TODO: Handle extension
+			 */
+			proto_tree_add_text(parent_tree, tvb, (offset>>3), 1, "dissect_per_constrained_sequence_of with extension is not handled");
+		}
+	}
+
 	/* 19.5 if min==max and min,max<64k ==> no length determinant */
 	if((min_len==max_len) && (min_len<65536)){
 		length=min_len;
@@ -841,11 +893,11 @@ call_sohelper:
 
 /* this function dissects a constrained set of */
 guint32
-dissect_per_constrained_set_of(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *parent_tree, int hf_index, gint ett_index, const per_sequence_t *seq, int min_len, int max_len)
+dissect_per_constrained_set_of(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *parent_tree, int hf_index, gint ett_index, const per_sequence_t *seq, int min_len, int max_len, gboolean has_extension)
 {
 	/* for basic-per  a set-of is encoded in the same way as a sequence-of */
 DEBUG_ENTRY("dissect_per_constrained_set_of");
-	offset=dissect_per_constrained_sequence_of(tvb, offset, actx, parent_tree, hf_index, ett_index, seq, min_len, max_len);
+	offset=dissect_per_constrained_sequence_of(tvb, offset, actx, parent_tree, hf_index, ett_index, seq, min_len, max_len, has_extension);
 	return offset;
 }
 
@@ -900,18 +952,18 @@ DEBUG_ENTRY("dissect_per_object_identifier");
 }
 
 guint32
-dissect_per_object_identifier_str(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *tree, int hf_index, const char **value_string)
+dissect_per_object_identifier_str(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *tree, int hf_index, const char **value_stringx)
 {
   tvbuff_t *value_tvb = NULL;
   guint length;
 
-  offset = dissect_per_object_identifier(tvb, offset, actx, tree, hf_index, (value_string) ? &value_tvb : NULL);
+  offset = dissect_per_object_identifier(tvb, offset, actx, tree, hf_index, (value_stringx) ? &value_tvb : NULL);
 
-  if (value_string) {
+  if (value_stringx) {
     if (value_tvb && (length = tvb_length(value_tvb))) {
-      *value_string = oid_encoded2string(tvb_get_ptr(value_tvb, 0, length), length);
+      *value_stringx = oid_encoded2string(tvb_get_ptr(value_tvb, 0, length), length);
     } else {
-      *value_string = "";
+      *value_stringx = "";
     }
   }
 
@@ -953,7 +1005,7 @@ DEBUG_ENTRY("dissect_per_boolean");
 			hfi->name,
 			value?"True":"False"
 		);
-		actx->created_item = proto_tree_add_boolean_format(tree, hf_index, tvb, offset>>3, 1, value, str);
+		actx->created_item = proto_tree_add_boolean_format(tree, hf_index, tvb, offset>>3, 1, value, "%s", str);
 	} else {
 		actx->created_item = NULL;
 	}
@@ -977,10 +1029,10 @@ dissect_per_integer(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree
 	header_field_info *hfi;
 
 	/* 12.2.6 b */
-	offset=dissect_per_length_determinant(tvb, offset, actx, tree, -1, &length);
+	offset=dissect_per_length_determinant(tvb, offset, actx, tree,hf_per_integer_length, &length);
 	/* gassert here? */
 	if(length>4){
-PER_NOT_DECODED_YET("too long integer");
+PER_NOT_DECODED_YET("too long integer(per_integer)");
 		length=4;
 	}
 
@@ -1033,7 +1085,7 @@ dissect_per_integer64b(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tr
 	offset=dissect_per_length_determinant(tvb, offset, actx, tree, -1, &length);
 	/* gassert here? */
 	if(length>8){
-PER_NOT_DECODED_YET("too long integer");
+PER_NOT_DECODED_YET("too long integer (64b)");
 		length=4;
 	}
 
@@ -1290,7 +1342,7 @@ DEBUG_ENTRY("dissect_per_constrained_integer");
 guint32
 dissect_per_constrained_integer_64b(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *tree, int hf_index, guint64 min, guint64 max, guint64 *value, gboolean has_extension)
 {
-	proto_item *it=NULL;
+	proto_item *it=NULL, *int_item=NULL;
 	guint64 range, val;
 	gint val_start, val_length;
 	nstime_t timeval;
@@ -1317,9 +1369,12 @@ DEBUG_ENTRY("dissect_per_constrained_integer_64b");
 	 *			d)	"range" is greater than 64K (the indefinite length case).
 	 */
 	if(((max-min)>65536)&&(actx->aligned)){
-		/* just set range really big so it will fall through
-		   to the bottom of the encoding */
-		range=1000000;
+               /* just set range really big so it will fall through
+                  to the bottom of the encoding */
+               /* range=1000000; */		
+			   range = max-min;
+			   if (range==65536)
+				   range++; /* make it fall trough? */
 	} else {
 		/* Copied from the 32 bit version, asuming the same problem occures
 		 * at 64 bit boundary.
@@ -1443,20 +1498,23 @@ DEBUG_ENTRY("dissect_per_constrained_integer_64b");
 		val_start = (offset>>3)-2; val_length = 2;
 		val+=min;
 	} else {
-		int i,num_bytes;
-		gboolean bit;
+		int i,num_bytes,num_bits;
 
 		/* 10.5.7.4 */
 		/* 12.2.6 */
-		offset=dissect_per_boolean(tvb, offset, actx, tree, -1, &bit);
-		num_bytes=bit;
-		offset=dissect_per_boolean(tvb, offset, actx, tree, -1, &bit);
-		num_bytes=(num_bytes<<1)|bit;
-
+		/* calculate the number of bits to hold the length */
+		if ((range & G_GINT64_CONSTANT(0xffffffff0000000)) != 0){
+			num_bits=3;
+		}else{
+			num_bits=2;
+		}
+		num_bytes =tvb_get_bits8(tvb, offset, num_bits);
 		num_bytes++;  /* lower bound for length determinant is 1 */
-		if (display_internal_per_fields)
-			proto_tree_add_uint(tree, hf_per_const_int_len, tvb, (offset>>3), 1, num_bytes);
-
+		if (display_internal_per_fields){
+			int_item = proto_tree_add_bits_item(tree, hf_per_const_int_len, tvb, offset,num_bits, FALSE);
+			proto_item_append_text(int_item,"+1=%u bytes, Range = (%" G_GINT64_MODIFIER "u)",num_bytes, range);
+		}
+		offset = offset+num_bits;
 		/* byte aligned */
 		BYTE_ALIGN_OFFSET(offset);
 		val=0;
@@ -1864,6 +1922,7 @@ DEBUG_ENTRY("dissect_per_sequence");
 
 			if(sequence[extension_index].func){
 				new_offset=sequence[extension_index].func(tvb, offset, actx, tree, *sequence[extension_index].p_id);
+				if (new_offset == offset) new_offset += 8;  /* OpenType has at least 1 octet */
 				offset+=length*8;
 				difference = offset - new_offset;
 				/* A difference of 7 or less might be byte aligning */
@@ -1962,6 +2021,7 @@ DEBUG_ENTRY("dissect_per_bit_string");
 			BYTE_ALIGN_OFFSET(offset);
 		}
 		out_tvb = new_octet_aligned_subset_bits(tvb, offset, actx, length);
+
 		if (hfi) {
 			actx->created_item = proto_tree_add_item(tree, hf_index, out_tvb, 0, -1, FALSE);
 			proto_item_append_text(actx->created_item, " [bit length %u]", length);
@@ -2368,7 +2428,7 @@ proto_register_per(void)
 		{ "Open Type Length", "per.open_type_length", FT_UINT32, BASE_DEC,
 		NULL, 0, "Length of an open type encoding", HFILL }},
 	{ &hf_per_real_length,
-		{ "Raal Length", "per.real_length", FT_UINT32, BASE_DEC,
+		{ "Real Length", "per.real_length", FT_UINT32, BASE_DEC,
 		NULL, 0, "Length of an real encoding", HFILL }},
 	{ &hf_per_octet_string_length,
 		{ "Octet String Length", "per.octet_string_length", FT_UINT32, BASE_DEC,
@@ -2407,6 +2467,10 @@ proto_register_per(void)
       { "arbitrary", "per.arbitrary",
         FT_BYTES, BASE_HEX, NULL, 0,
         "per.T_arbitrary", HFILL }},
+    { &hf_per_integer_length,
+      { "integer length", "per.integer_length",
+        FT_UINT32, BASE_DEC, NULL, 0,
+        NULL, HFILL }},
 	};
 	static gint *ett[] =
 	{
