@@ -10,7 +10,7 @@
  * Dinesh G Dutt (ddutt@cisco.com)
  * Ronnie sahlberg 2006
  *
- * $Id: packet-scsi-ssc.c 24643 2008-03-15 22:41:57Z morriss $
+ * $Id: packet-scsi-ssc.c 26091 2008-08-26 01:28:50Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -39,8 +39,6 @@
 #include <string.h>
 #include <epan/strutil.h>
 #include <epan/packet.h>
-#include <epan/prefs.h>
-#include <epan/emem.h>
 #include <epan/conversation.h>
 #include <epan/tap.h>
 #include "packet-scsi.h"
