@@ -1,7 +1,7 @@
 /* filesystem.c
  * Filesystem utility routines
  *
- * $Id: filesystem.c 27718 2009-03-13 22:06:48Z stig $
+ * $Id: filesystem.c 27821 2009-03-22 18:06:06Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -26,6 +26,13 @@
 # include "config.h"
 #endif
 
+/*
+ * Required with GNU libc to get dladdr().
+ * We define it here because <dlfcn.h> apparently gets included by
+ * one of the headers we include below.
+ */
+#define _GNU_SOURCE
+
 #ifdef HAVE_DIRENT_H
 #include <dirent.h>
 #endif
@@ -55,9 +62,12 @@
 #include <tchar.h>
 #include <shlobj.h>
 #include <wsutil/unicode-utils.h>
-#else
+#else /* _WIN32 */
+#ifdef DLADDR_FINDS_EXECUTABLE_PATH
+#include <dlfcn.h>
+#endif /* DLADDR_FINDS_EXECUTABLE_PATH */
 #include <pwd.h>
-#endif
+#endif /* _WIN32 */
 
 #include "filesystem.h"
 #include "report_err.h"
@@ -246,6 +256,10 @@ init_progfile_dir(const char *arg0
 #ifdef _WIN32
 	_U_
 #endif
+, int (*main_addr)(int, char **)
+#if defined(_WIN32) || !defined(DLADDR_FINDS_EXECUTABLE_PATH)
+	_U_
+#endif
 )
 {
 	char *dir_end;
@@ -334,6 +348,9 @@ init_progfile_dir(const char *arg0
 		    msg, error);
 	}
 #else
+#ifdef DLADDR_FINDS_EXECUTABLE_PATH
+	Dl_info info;
+#endif
 	char *prog_pathname;
 	char *curdir;
 	long path_max;
@@ -355,10 +372,24 @@ init_progfile_dir(const char *arg0
 	    && !started_with_special_privs())
 		running_in_build_directory_flag = TRUE;
 
+#ifdef DLADDR_FINDS_EXECUTABLE_PATH
+	/*
+	 * Try to use dladdr() to find the pathname of the executable.
+	 * dladdr() is not guaranteed to give you anything better than
+	 * argv[0] (i.e., it might not contain a / at all, much less
+	 * being an absolute path), and doesn't appear to do so on
+	 * Linux, but on other platforms it could give you an absolute
+	 * path and obviate the need for us to determine the absolute
+	 * path.
+	 */
+	if (dladdr((void *)main_addr, &info))
+		arg0 = info.dli_fname;
+#endif
 	/*
 	 * Try to figure out the directory in which the currently running
-	 * program resides, given the argv[0] it was started with.  That
-	 * might be the absolute path of the program, or a path relative
+	 * program resides, given something purporting to be the executable
+	 * name (from dladdr() or from the argv[0] it was started with.
+	 * That might be the absolute path of the program, or a path relative
 	 * to the current directory of the process that started it, or
 	 * just a name for the program if it was started from the command
 	 * line and was searched for in $PATH.  It's not guaranteed to be
