@@ -3,7 +3,7 @@
  *
  * (c) 2007, Luis E. Garcia Ontanon <luis@ontanon.org>
  *
- * $Id: oids.c 26113 2008-09-02 11:17:03Z wmeier $
+ * $Id: oids.c 27753 2009-03-17 15:34:18Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -70,7 +70,7 @@ static const oid_value_type_t counter64_type =  { FT_UINT64, BASE_DEC,  BER_CLAS
 static const oid_value_type_t ipv6_type =       { FT_IPv6,   BASE_NONE, BER_CLASS_UNI, BER_UNI_TAG_OCTETSTRING, 16, 16, OID_KEY_TYPE_BYTES,   16};
 static const oid_value_type_t float_type =      { FT_FLOAT,  BASE_DEC,  BER_CLASS_UNI, BER_UNI_TAG_OCTETSTRING, 4,   4, OID_KEY_TYPE_WRONG,   0};
 static const oid_value_type_t double_type =     { FT_DOUBLE, BASE_DEC,  BER_CLASS_UNI, BER_UNI_TAG_OCTETSTRING, 8,   8, OID_KEY_TYPE_WRONG,   0};
-static const oid_value_type_t ether_type =      { FT_ETHER,  BASE_NONE, BER_CLASS_UNI, BER_UNI_TAG_OCTETSTRING, 6,   6, OID_KEY_TYPE_BYTES,   6};
+static const oid_value_type_t ether_type =      { FT_ETHER,  BASE_NONE, BER_CLASS_UNI, BER_UNI_TAG_OCTETSTRING, 6,   6, OID_KEY_TYPE_ETHER,   6};
 static const oid_value_type_t string_type =     { FT_STRING, BASE_NONE, BER_CLASS_UNI, BER_UNI_TAG_OCTETSTRING, 0,  -1, OID_KEY_TYPE_STRING,  0};
 static const oid_value_type_t unknown_type =    { FT_BYTES,  BASE_NONE, BER_CLASS_ANY, BER_TAG_ANY,             0,  -1, OID_KEY_TYPE_WRONG,   0};
 
@@ -187,22 +187,22 @@ extern void oid_add_from_encoded(const char* name, const guint8 *oid, gint oid_l
 #ifdef HAVE_LIBSMI
 /* de-allocate storage mallocated by libsmi                            */
 /*                                                                     */
-/* XXX: On Windows we can use free() only if the Windows libsmi.dll    */
-/*      being used is linked with the same CRTL as wireshark.          */
-/*      As a temporary hack we'll assume on Windows that the Wireshark */
-/*      build of libsmi.dll was done with VC6/msvcrt.dll (as is        */
-/*      currently the case).  If wireshark is being built with         */
-/*      vc6 (which is also currently the case for the standard         */
-/*      wireshark release). we can use free().                         */
-/*      If vc6 is not being used here, then we'll just have to live    */
-/*      with a memory leak for now.                                    */
-/*      Note: A permanent fix would probably be for libsmi to include  */
-/*       its' existing smiFree function as part of the libsmi API so   */
-/*       wireshark can call it to free storage mallocated by libsmi.   */
+/* XXX: libsmi provides access to smiFree as of libsmi v 0.4.8.        */
+/*      On Windows: Wireshark 1.01 and later is built and distributed  */
+/*      with libsmi 0.4.8 (or newer).                                  */
+/*      On non-Windows systems, free() should be OK for libsmi         */
+/*       versions older than 0.4.8.                                    */
 
 static void smi_free(void *ptr) {
-#if !defined _WIN32 || (_MSC_VER == 1200)
-	free(ptr);
+
+#if (SMI_VERSION_MAJOR >= 0) && (SMI_VERSION_MINOR >= 4) && (SMI_VERSION_PATCHLEVEL >= 8)
+       smiFree(ptr);
+#else
+ #ifdef _WIN32
+ #error Invalid Windows libsmi version ?? !!
+ #endif
+#define xx_free free  /* hack so checkAPIs.pl doesn't complain */
+       xx_free(ptr);
 #endif
 }
 
@@ -243,7 +243,7 @@ static void* smi_mod_copy_cb(void* dest, const void* orig, unsigned len _U_) {
 
 static void smi_mod_free_cb(void* p) {
 	smi_module_t* m = p;
-	if (m->name) g_free(m->name);
+	g_free(m->name);
 }
 
 
@@ -494,11 +494,11 @@ void register_mibs(void) {
 	GArray* hfa = g_array_new(FALSE,TRUE,sizeof(hf_register_info));
 	GArray* etta = g_array_new(FALSE,TRUE,sizeof(gint*));
 	static uat_field_t smi_fields[] = {
-		UAT_FLD_CSTRING(smi_mod,name,"The module's name"),
+		UAT_FLD_CSTRING(smi_mod,name,"Module name","The module's name"),
 		UAT_END_FIELDS
 	};
 	static uat_field_t smi_paths_fields[] = {
-		UAT_FLD_CSTRING(smi_mod,name,"The directory name"),
+		UAT_FLD_PATHNAME(smi_mod,name,"Directory path","The directory name"),
 		UAT_END_FIELDS
 	};
 	char* smi_load_error = NULL;
@@ -730,7 +730,7 @@ const char* oid_subid2string(guint32* subids, guint len) {
 		return "*** Empty OID ***";
 
 	do {
-		w += sprintf(w,"%u.",*subids++);
+		w += g_snprintf(w,12,"%u.",*subids++);
 	} while(--len);
 
 	if (w!=s) *(w-1) = '\0'; else *(s) = '\0';
