@@ -1,6 +1,6 @@
 /* decode_as_dlg.c
  *
- * $Id: decode_as_dlg.c 26958 2008-12-10 11:04:36Z kukosa $
+ * $Id: decode_as_dlg.c 28269 2009-05-04 21:13:14Z stig $
  *
  * Routines to modify dissector tables on the fly.
  *
@@ -37,6 +37,7 @@
 #include "../globals.h"
 #include "../simple_dialog.h"
 
+#include "gtk/main.h"
 #include "gtk/decode_as_dlg.h"
 #include "gtk/dlg_utils.h"
 #include "gtk/gui_utils.h"
@@ -428,7 +429,7 @@ decode_clear_all(void)
 
     decode_dcerpc_reset_all();
 
-    cf_redissect_packets(&cfile);
+    redissect_packets();
 }
 
 
@@ -652,8 +653,7 @@ decode_change_one_dissector(gchar *table_name, guint selector, GtkWidget *list)
     } else {
 	dissector_change(table_name, selector, handle);
     }
-    if (abbrev != NULL)
-	g_free(abbrev);
+    g_free(abbrev);
 }
 
 
@@ -838,7 +838,7 @@ decode_ok_cb (GtkWidget *ok_bt _U_, gpointer parent_w)
     g_slist_free(decode_dimmable);
     decode_dimmable = NULL;
 
-    cf_redissect_packets(&cfile);
+    redissect_packets();
 }
 
 /*
@@ -867,7 +867,7 @@ decode_apply_cb (GtkWidget *apply_bt _U_, gpointer parent_w)
     func = g_object_get_data(G_OBJECT(notebook_pg), E_PAGE_ACTION);
     func(notebook_pg);
 
-    cf_redissect_packets(&cfile);
+    redissect_packets();
 }
 
 /*
@@ -1029,7 +1029,7 @@ decode_add_pack_menu (GtkWidget *optmenu)
 {
     GtkWidget *alignment;
 
-    alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
+    alignment = gtk_alignment_new(0.0f, 0.5f, 0.0f, 0.0f);
     gtk_container_add(GTK_CONTAINER(alignment), optmenu);
 
     return(alignment);
@@ -1056,13 +1056,13 @@ decode_add_srcdst_menu (GtkWidget *page)
 
     optmenu = gtk_option_menu_new();
     menu = gtk_menu_new();
-    g_snprintf(tmp, 100, "source (%u)", cfile.edt->pi.srcport);
+    g_snprintf(tmp, sizeof(tmp), "source (%u)", cfile.edt->pi.srcport);
     menuitem = gtk_menu_item_new_with_label(tmp);
     g_object_set_data(G_OBJECT(menuitem), "user_data", GINT_TO_POINTER(E_DECODE_SPORT));
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
     gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
 
-    g_snprintf(tmp, 100, "destination (%u)", cfile.edt->pi.destport);
+    g_snprintf(tmp, sizeof(tmp), "destination (%u)", cfile.edt->pi.destport);
     menuitem = gtk_menu_item_new_with_label(tmp);
     g_object_set_data(G_OBJECT(menuitem), "user_data", GINT_TO_POINTER(E_DECODE_DPORT));
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
@@ -1093,7 +1093,7 @@ decode_add_ppid_menu (GtkWidget *page)
     optmenu = gtk_option_menu_new();
     menu = gtk_menu_new();
     
-    g_snprintf(tmp, 100, "PPID (%u)", 0);
+    g_snprintf(tmp, sizeof(tmp), "PPID (%u)", 0);
     menuitem = gtk_menu_item_new_with_label(tmp);
     g_object_set_data(G_OBJECT(menuitem), "user_data", GINT_TO_POINTER(E_DECODE_PPID));
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
@@ -1101,13 +1101,13 @@ decode_add_ppid_menu (GtkWidget *page)
     
     for(number_of_ppid = 0; number_of_ppid < MAX_NUMBER_OF_PPIDS; number_of_ppid++)
       if (cfile.edt->pi.ppids[number_of_ppid] != 0) {
-        g_snprintf(tmp, 100, "PPID (%u)", cfile.edt->pi.ppids[number_of_ppid]);
-        menuitem = gtk_menu_item_new_with_label(tmp);
-        g_object_set_data(G_OBJECT(menuitem), "user_data", GINT_TO_POINTER(E_DECODE_PPID + 1 + number_of_ppid));
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-        gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
+          g_snprintf(tmp, sizeof(tmp), "PPID (%u)", cfile.edt->pi.ppids[number_of_ppid]);
+          menuitem = gtk_menu_item_new_with_label(tmp);
+          g_object_set_data(G_OBJECT(menuitem), "user_data", GINT_TO_POINTER(E_DECODE_PPID + 1 + number_of_ppid));
+          gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+          gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
       } else
-        break;
+          break;
 
     g_object_set_data(G_OBJECT(page), E_MENU_SRCDST, menu);
     gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), menu);
@@ -1492,14 +1492,14 @@ decode_sctp_update_ppid_menu(GtkWidget *w _U_, GtkWidget *page)
 
     menu = gtk_menu_new();
 
-    g_snprintf(tmp, 100, "PPID (%u)", 0);
+    g_snprintf(tmp, sizeof(tmp), "PPID (%u)", 0);
     menuitem = gtk_menu_item_new_with_label(tmp);
     g_object_set_data(G_OBJECT(menuitem), "user_data", GINT_TO_POINTER(E_DECODE_PPID));
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
     gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
     for(number_of_ppid = 0; number_of_ppid < MAX_NUMBER_OF_PPIDS; number_of_ppid++)
       if (cfile.edt->pi.ppids[number_of_ppid] != 0) {
-        g_snprintf(tmp, 100, "PPID (%u)", cfile.edt->pi.ppids[number_of_ppid]);
+        g_snprintf(tmp, sizeof(tmp), "PPID (%u)", cfile.edt->pi.ppids[number_of_ppid]);
         menuitem = gtk_menu_item_new_with_label(tmp);
         g_object_set_data(G_OBJECT(menuitem), "user_data", GINT_TO_POINTER(E_DECODE_PPID + 1 + number_of_ppid));
         gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
@@ -1528,13 +1528,13 @@ decode_sctp_update_srcdst_menu(GtkWidget *w _U_, GtkWidget *page)
     GtkListStore *sctp_store;
 
     menu = gtk_menu_new();
-    g_snprintf(tmp, 100, "source (%u)", cfile.edt->pi.srcport);
+    g_snprintf(tmp, sizeof(tmp), "source (%u)", cfile.edt->pi.srcport);
     menuitem = gtk_menu_item_new_with_label(tmp);
     g_object_set_data(G_OBJECT(menuitem), "user_data", GINT_TO_POINTER(E_DECODE_SPORT));
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
     gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
 
-    g_snprintf(tmp, 100, "destination (%u)", cfile.edt->pi.destport);
+    g_snprintf(tmp, sizeof(tmp), "destination (%u)", cfile.edt->pi.destport);
     menuitem = gtk_menu_item_new_with_label(tmp);
     g_object_set_data(G_OBJECT(menuitem), "user_data", GINT_TO_POINTER(E_DECODE_DPORT));
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
@@ -1659,7 +1659,7 @@ decode_add_notebook (GtkWidget *format_hb)
 
     /* Add link level selection page */
     if (cfile.edt->pi.ethertype) {
-	g_snprintf(buffer, 40, "Ethertype 0x%04x", cfile.edt->pi.ethertype);
+	g_snprintf(buffer, sizeof(buffer), "Ethertype 0x%04x", cfile.edt->pi.ethertype);
 	page = decode_add_simple_page(buffer, "Link", "ethertype", cfile.edt->pi.ethertype);
 	label = gtk_label_new("Link");
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, label);
@@ -1667,7 +1667,7 @@ decode_add_notebook (GtkWidget *format_hb)
 
     /* Add mpls selection page */
     if (cfile.edt->pi.mpls_label) {
-	g_snprintf(buffer, 40, "Data after label %u", cfile.edt->pi.mpls_label);
+	g_snprintf(buffer, sizeof(buffer), "Data after label %u", cfile.edt->pi.mpls_label);
 	page = decode_add_simple_page(buffer, "MPLS", "mpls.label", cfile.edt->pi.mpls_label);
 	label = gtk_label_new("MPLS");
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, label);
@@ -1678,7 +1678,7 @@ decode_add_notebook (GtkWidget *format_hb)
 	/*
 	 * The network-layer protocol is IP.
 	 */
-	g_snprintf(buffer, 40, "IP protocol %u", cfile.edt->pi.ipproto);
+	g_snprintf(buffer, sizeof(buffer), "IP protocol %u", cfile.edt->pi.ipproto);
 	page = decode_add_simple_page(buffer, "Network", "ip.proto", cfile.edt->pi.ipproto);
 	g_object_set_data(G_OBJECT(page), E_PAGE_ACTION, decode_simple);
 	label = gtk_label_new("Network");
