@@ -1,7 +1,7 @@
 /* follow_stream.c
  * Common routines for following data streams
  *
- * $Id: follow_stream.c 25634 2008-06-29 15:51:43Z wmeier $
+ * $Id: follow_stream.c 28662 2009-06-08 15:37:46Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -56,8 +56,11 @@
 #include <gtk/gui_utils.h>
 #include <gtk/help_dlg.h>
 #include "gtk/main.h"
-#include "gtk/print_win32.h"
 
+#ifdef _WIN32
+#include "../tempfile.h"
+#include "gtk/print_win32.h"
+#endif
 
 /* static variable declarations to speed up the performance
  * of follow_load_text and follow_add_to_gtk_text
@@ -66,7 +69,11 @@ static GdkColor server_fg, server_bg;
 static GdkColor client_fg, client_bg;
 static GtkTextTag *server_tag, *client_tag;
 
+static void follow_find_destroy_cb(GtkWidget * win _U_, gpointer data);
+static void follow_find_button_cb(GtkWidget * w, gpointer data);
+static gboolean follow_save_as_ok_cb(GtkWidget * w _U_, gpointer fs);
 static void follow_destroy_cb(GtkWidget *w, gpointer data _U_);
+static void follow_save_as_destroy_cb(GtkWidget * win _U_, gpointer data);
 
 GList *follow_infos = NULL;
 
@@ -101,7 +108,7 @@ follow_add_to_gtk_text(char *buffer, size_t nchars, gboolean is_server,
 	GtkTextIter    iter;
 
 	/* While our isprint() hack is in place, we
-	 * have to use convert some chars to '.' in order
+	 * have to convert some chars to '.' in order
 	 * to be able to see the data we *should* see
 	 * in the GtkText widget.
 	 */
@@ -117,10 +124,10 @@ follow_add_to_gtk_text(char *buffer, size_t nchars, gboolean is_server,
 
 	gtk_text_buffer_get_end_iter(buf, &iter);
 	if (is_server) {
-		gtk_text_buffer_insert_with_tags(buf, &iter, buffer, nchars,
+		gtk_text_buffer_insert_with_tags(buf, &iter, buffer, (gint) nchars,
 						 server_tag, NULL);
 	} else {
-		gtk_text_buffer_insert_with_tags(buf, &iter, buffer, nchars,
+		gtk_text_buffer_insert_with_tags(buf, &iter, buffer, (gint) nchars,
 						 client_tag, NULL);
 	}
 	return TRUE;
@@ -174,7 +181,7 @@ follow_write_raw(char *buffer, size_t nchars, gboolean is_server _U_, void *arg)
 }
 
 /* Handles the display style toggling */
-void
+static void
 follow_charset_toggle_cb(GtkWidget * w _U_, gpointer data)
 {
 	follow_info_t	*follow_info = data;
@@ -251,7 +258,7 @@ follow_filter_out_stream(GtkWidget * w _U_, gpointer data)
 	return;
 }
 
-void
+static void
 follow_find_cb(GtkWidget * w _U_, gpointer data)
 {
 	follow_info_t      	*follow_info = data;
@@ -321,7 +328,7 @@ follow_find_cb(GtkWidget * w _U_, gpointer data)
 	window_present(find_dlg_w);
 }
 
-void
+static void
 follow_find_button_cb(GtkWidget * w, gpointer data)
 {
 	gboolean		found;
@@ -369,7 +376,7 @@ follow_find_button_cb(GtkWidget * w, gpointer data)
 
 }
 
-void
+static void
 follow_find_destroy_cb(GtkWidget * win _U_, gpointer data)
 {
 	follow_info_t	*follow_info = data;
@@ -378,23 +385,40 @@ follow_find_destroy_cb(GtkWidget * win _U_, gpointer data)
 	follow_info->find_dlg_w = NULL;
 }
 
-void
+static void
 follow_print_stream(GtkWidget * w _U_, gpointer data)
 {
 	print_stream_t	*stream;
-	gboolean		to_file;
+	gboolean	 to_file;
 	char		*print_dest;
 	follow_info_t	*follow_info = data;
 #ifdef _WIN32
-	gboolean win_printer = FALSE;
+	gboolean         win_printer = FALSE;
+	int              tmp_fd;
+	char             tmp_namebuf[128+1];  /* see create_tmpfile which says [128+1]; why ? */
 #endif
 
 	switch (prefs.pr_dest) {
 	case PR_DEST_CMD:
 #ifdef _WIN32
 		win_printer = TRUE;
-		/*XXX should use temp file stuff in util routines */
-		print_dest = g_strdup(tmpnam(NULL));
+		/* (The code for creating a temp filename is adapted from print_dlg.c).   */
+		/* We currently don't have a function in util.h to create just a tempfile */
+		/* name, so simply create a tempfile using the "official" function,       */
+		/* then delete this file again. After this, the name MUST be available.   */
+		/* */
+		/* Don't use tmpnam() or such, as this will fail under some ACL           */
+		/* circumstances: http://bugs.wireshark.org/bugzilla/show_bug.cgi?id=358  */
+		/* Also: tmpnam is "insecure" and should not be used.                     */
+		tmp_fd = create_tempfile(tmp_namebuf, sizeof(tmp_namebuf), "wshprint");
+		if(tmp_fd == -1) {
+			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+				      "Couldn't create a temporary file for printing.");
+			return;
+		}
+		ws_close(tmp_fd);
+		ws_unlink(tmp_namebuf);
+		print_dest = tmp_namebuf;
 		to_file = TRUE;
 #else
 		print_dest = prefs.pr_cmd;
@@ -428,7 +452,7 @@ follow_print_stream(GtkWidget * w _U_, gpointer data)
 	}
 	if (stream == NULL) {
 		if (to_file) {
-			open_failure_alert_box(prefs.pr_file, errno, TRUE);
+			open_failure_alert_box(print_dest, errno, TRUE);
 		} else {
 			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 				      "Couldn't run print command %s.",
@@ -457,7 +481,7 @@ follow_print_stream(GtkWidget * w _U_, gpointer data)
 
 	if (!destroy_print_stream(stream)) {
 		if (to_file) {
-			write_failure_alert_box(prefs.pr_file, errno);
+			write_failure_alert_box(print_dest, errno);
 		} else {
 			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 				      "Error closing print destination.");
@@ -475,7 +499,7 @@ follow_print_stream(GtkWidget * w _U_, gpointer data)
 
  print_error:
 	if (to_file) {
-		write_failure_alert_box(prefs.pr_file, errno);
+		write_failure_alert_box(print_dest, errno);
 	} else {
 		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 			      "Error writing to print command: %s",
@@ -498,21 +522,26 @@ follow_print_stream(GtkWidget * w _U_, gpointer data)
  * while there's already a "Save Follow Stream" window up, we just pop
  * up the existing one, rather than creating a new one.
  */
-void
+
+static void
 follow_save_as_cmd_cb(GtkWidget *w _U_, gpointer data)
 {
 	GtkWidget		*new_win;
 	follow_info_t	*follow_info = data;
 
+#if 0  /* XXX: GtkFileChooserDialog/gtk_dialog_run currently being used is effectively modal so this is not req'd */
 	if (follow_info->follow_save_as_w != NULL) {
 		/* There's already a dialog box; reactivate it. */
 		reactivate_window(follow_info->follow_save_as_w);
 		return;
 	}
-
+#endif
 	new_win = file_selection_new("Wireshark: Save Follow Stream As",
 				     FILE_SELECTION_SAVE);
 	follow_info->follow_save_as_w = new_win;
+#if GTK_CHECK_VERSION(2,8,0)
+	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(new_win), TRUE);
+#endif
 
 	/* Tuck away the follow_info object into the window */
 	g_object_set_data(G_OBJECT(new_win), E_FOLLOW_INFO_KEY, follow_info);
@@ -520,16 +549,37 @@ follow_save_as_cmd_cb(GtkWidget *w _U_, gpointer data)
 	g_signal_connect(new_win, "destroy", G_CALLBACK(follow_save_as_destroy_cb),
 		       follow_info);
 
+#if 0
 	if (gtk_dialog_run(GTK_DIALOG(new_win)) == GTK_RESPONSE_ACCEPT)
 		{
 			follow_save_as_ok_cb(new_win, new_win);
 		} else {
 		window_destroy(new_win);
 	}
+#endif
+	/* "Run" the GtkFileChooserDialog.                                              */
+        /* Upon exit: If "Accept" run the OK callback.                                  */
+        /*            If the OK callback returns with a FALSE status, re-run the dialog.*/
+        /*            If not accept (ie: cancel) destroy the window.                    */
+        /* XXX: If the OK callback pops up an alert box (eg: for an error) it *must*    */
+        /*      return with a TRUE status so that the dialog window will be destroyed.  */
+	/*      Trying to re-run the dialog after popping up an alert box will not work */
+        /*       since the user will not be able to dismiss the alert box.              */
+	/*      The (somewhat unfriendly) effect: the user must re-invoke the           */
+	/*      GtkFileChooserDialog whenever the OK callback pops up an alert box.     */
+	/*                                                                              */
+        /*      ToDo: use GtkFileChooserWidget in a dialog window instead of            */
+	/*            GtkFileChooserDialog.                                             */
+	while (gtk_dialog_run(GTK_DIALOG(new_win)) == GTK_RESPONSE_ACCEPT) {
+		if (follow_save_as_ok_cb(NULL, new_win)) {
+                    break; /* we're done */
+		}
+	}
+	window_destroy(new_win);
 }
 
 
-void
+static gboolean
 follow_save_as_ok_cb(GtkWidget * w _U_, gpointer fs)
 {
 	gchar		*to_name;
@@ -538,7 +588,7 @@ follow_save_as_ok_cb(GtkWidget * w _U_, gpointer fs)
 	print_stream_t	*stream = NULL;
 	gchar		*dirname;
 
-	to_name = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fs)));
+	to_name = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fs));
 
 	/* Perhaps the user specified a directory instead of a file.
 	   Check whether they did. */
@@ -548,10 +598,12 @@ follow_save_as_ok_cb(GtkWidget * w _U_, gpointer fs)
 		set_last_open_dir(to_name);
 		g_free(to_name);
 		file_selection_set_current_folder(fs, get_last_open_dir());
-		return;
+		gtk_file_chooser_set_current_name(fs, "");
+		return FALSE; /* do gtk_dialog_run again */
 	}
 
 	follow_info = g_object_get_data(G_OBJECT(fs), E_FOLLOW_INFO_KEY);
+
 	if (follow_info->show_type == SHOW_RAW) {
 		/* Write the data out as raw binary data */
 		fh = ws_fopen(to_name, "wb");
@@ -562,12 +614,13 @@ follow_save_as_ok_cb(GtkWidget * w _U_, gpointer fs)
 	if (fh == NULL) {
 		open_failure_alert_box(to_name, errno, TRUE);
 		g_free(to_name);
-		return;
+		return TRUE;
 	}
 
+#if 0 /* handled by caller (for now) .... */
 	gtk_widget_hide(GTK_WIDGET(fs));
 	window_destroy(GTK_WIDGET(fs));
-
+#endif
 	if (follow_info->show_type == SHOW_RAW) {
 		switch (follow_read_stream(follow_info, follow_write_raw, fh)) {
 		case FRS_OK:
@@ -610,9 +663,10 @@ follow_save_as_ok_cb(GtkWidget * w _U_, gpointer fs)
 	dirname = get_dirname(to_name);  /* Overwrites to_name */
 	set_last_open_dir(dirname);
 	g_free(to_name);
+        return TRUE;
 }
 
-void
+static void
 follow_save_as_destroy_cb(GtkWidget * win _U_, gpointer data)
 {
 	follow_info_t	*follow_info = data;
@@ -644,7 +698,7 @@ follow_stream_direction_changed(GtkWidget *w, gpointer data)
 }
 
 /* Add a "follow_info_t" structure to the list. */
-void
+static void
 remember_follow_info(follow_info_t *follow_info)
 {
 	follow_infos = g_list_append(follow_infos, follow_info);
@@ -652,7 +706,7 @@ remember_follow_info(follow_info_t *follow_info)
 
 #define IS_SHOW_TYPE(x) (follow_info->show_type == x ? 1 : 0)
 /* Remove a "follow_info_t" structure from the list. */
-void
+static void
 forget_follow_info(follow_info_t *follow_info)
 {
 	follow_infos = g_list_remove(follow_infos, follow_info);
@@ -754,12 +808,13 @@ follow_stream(gchar *title, follow_info_t *follow_info,
 	gtk_combo_box_append_text(GTK_COMBO_BOX(stream_cmb),
 				   client_to_server_string);
 
+	gtk_combo_box_set_active(GTK_COMBO_BOX(stream_cmb), 0); /* Do this before signal_connect  */
+								/*  so callback not triggered     */
+
 	g_signal_connect(stream_cmb, "changed",
 			 G_CALLBACK(follow_stream_direction_changed),
 			 follow_info);
 
-	gtk_combo_box_set_active(GTK_COMBO_BOX(stream_cmb), 0);
-
 	gtk_tooltips_set_tip (tooltips, stream_cmb,
 			      "Select the stream direction to display", NULL);
 	gtk_box_pack_start(GTK_BOX(hbox), stream_cmb, FALSE, FALSE, 0);
@@ -933,7 +988,7 @@ follow_show(follow_info_t *follow_info,
 
 	case SHOW_EBCDIC:
 		/* If our native arch is ASCII, call: */
-		EBCDIC_to_ASCII(buffer, nchars);
+		EBCDIC_to_ASCII(buffer, (guint) nchars);
 		if (!(*print_line) (buffer, nchars, is_server, arg))
 			return FRS_PRINT_ERROR;
 		break;
@@ -961,11 +1016,12 @@ follow_show(follow_info_t *follow_info,
 			int i;
 			gchar *cur = hexbuf, *ascii_start;
 
-			/* is_server indentation : put 78 spaces at the
+			/* is_server indentation : put 4 spaces at the
 			 * beginning of the string */
+			/* XXX - We might want to prepend each line with "C" or "S" instead. */
 			if (is_server && follow_info->show_stream == BOTH_HOSTS) {
-				memset(cur, ' ', 78);
-				cur += 78;
+				memset(cur, ' ', 4);
+				cur += 4;
 			}
 			cur += g_snprintf(cur, 20, "%08X  ", *global_pos);
 			/* 49 is space consumed by hex chars */
