@@ -5,7 +5,7 @@
  *
  * (c) 2006, Luis E. Garcia Ontanon <luis@ontanon.org>
  *
- * $Id: funnel_stat.c 27627 2009-03-06 22:49:27Z wmeier $
+ * $Id: funnel_stat.c 28108 2009-04-21 16:57:52Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -225,14 +225,14 @@ static void text_window_clear(funnel_text_window_t*  tw)
 static void text_window_append(funnel_text_window_t*  tw, const char *str)
 {
     GtkWidget *txt;
-    int nchars = strlen(str);
+    int nchars;
     GtkTextBuffer *buf;
     GtkTextIter    iter;
 
     if (! tw->win) return; 
 
     txt = tw->txt;
-    nchars = strlen(str);
+    nchars = (int) strlen(str);
     
     
     buf= gtk_text_view_get_buffer(GTK_TEXT_VIEW(txt));
@@ -258,14 +258,14 @@ static void text_window_set_text(funnel_text_window_t*  tw, const gchar* text)
 
 static void text_window_prepend(funnel_text_window_t*  tw, const char *str _U_) {
     GtkWidget *txt;
-    int nchars = strlen(str);
+    int nchars;
     GtkTextBuffer *buf;
     GtkTextIter    iter;
 	
     if (! tw->win) return; 
 	
     txt = tw->txt;
-    nchars = strlen(str);
+    nchars = (int) strlen(str);
     
     
     buf= gtk_text_view_get_buffer(GTK_TEXT_VIEW(txt));
@@ -432,7 +432,7 @@ static void funnel_new_dialog(const gchar* title,
         GtkWidget *entry, *label;
         
         label = gtk_label_new(fieldname);
-        gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+        gtk_misc_set_alignment(GTK_MISC(label), 1.0f, 0.5f);
         gtk_table_attach_defaults(GTK_TABLE(main_tb), label, 0, 1, i+1, i + 2);
         gtk_widget_show(label);
 
