@@ -6,7 +6,7 @@
  *
  * Modified from summary_dlg.c
  *
- * $Id: gsm_map_summary.c 26459 2008-10-14 22:55:16Z gerald $
+ * $Id: gsm_map_summary.c 28108 2009-04-21 16:57:52Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -66,7 +66,7 @@ add_string_to_box(gchar *str, GtkWidget *box)
 {
   GtkWidget *lb;
   lb = gtk_label_new(str);
-  gtk_misc_set_alignment(GTK_MISC(lb), 0.0, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(lb), 0.0f, 0.5f);
   gtk_box_pack_start(GTK_BOX(box), lb,FALSE,FALSE, 0);
   gtk_widget_show(lb);
 }
