@@ -3,7 +3,7 @@
  * and some with GUI-independent APIs, with this file containing the GTK+
  * implementations of them (declared in ui_util.h)
  *
- * $Id: gui_utils.c 26535 2008-10-24 00:42:09Z wmeier $
+ * $Id: gui_utils.c 28444 2009-05-22 12:36:20Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -29,6 +29,7 @@
 #endif
 
 #include <string.h>
+#include <locale.h>
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
@@ -415,7 +416,7 @@ window_geom_recent_read_pair(const char *name, const char *key, const char *valu
     /* find window geometry maybe already in hashtable */
     if(!window_geom_load(name, &geom)) {
         /* not in table, init geom with "basic" values */
-        geom.key        = g_strdup(name);
+        geom.key        = NULL;    /* Will be set in window_geom_save () */
         geom.set_pos    = FALSE;
         geom.x          = -1;
         geom.y          = -1;
@@ -536,8 +537,7 @@ set_main_window_name(const gchar *window_name)
 
   /* Attach the new un-decorated window name to the window. */
   old_window_name = g_object_get_data(G_OBJECT(top_level), MAIN_WINDOW_NAME_KEY);
-  if (old_window_name != NULL)
-    g_free(old_window_name);
+  g_free(old_window_name);
   g_object_set_data(G_OBJECT(top_level), MAIN_WINDOW_NAME_KEY, g_strdup(window_name));
 
   update_main_window_name();
@@ -570,7 +570,7 @@ void main_window_update(void)
 /* exit the main window */
 void main_window_exit(void)
 {
-  gtk_exit(0);
+  exit(0);
 }
 
 #ifdef HAVE_LIBPCAP
@@ -1094,6 +1094,7 @@ tree_view_key_pressed_cb(GtkWidget *tree, GdkEventKey *event, gpointer user_data
             if(expanded) {
                 /* subtree is expanded, collapse it */
                 gtk_tree_view_collapse_row(GTK_TREE_VIEW(tree), path);
+                gtk_tree_path_free(path);
                 return TRUE;
             }
             /* No break - fall through to jumping to the parent */
@@ -1103,6 +1104,7 @@ tree_view_key_pressed_cb(GtkWidget *tree, GdkEventKey *event, gpointer user_data
                 if(! gtk_tree_model_iter_parent(model, &parent, &iter)) {
                     return FALSE;
                 }
+                gtk_tree_path_free(path);
                 path = gtk_tree_model_get_path(model, &parent);
                 if(!path) {
                     return FALSE;
@@ -1110,12 +1112,14 @@ tree_view_key_pressed_cb(GtkWidget *tree, GdkEventKey *event, gpointer user_data
                 gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree), path,
                                                  NULL /* focus_column */,
                                                  FALSE /* !start_editing */);
+                gtk_tree_path_free(path);
                 return TRUE;
             }
             break;
         case GDK_Right:
             /* try to expand the subtree */
             gtk_tree_view_expand_row(GTK_TREE_VIEW(tree), path, FALSE /* !open_all */);
+            gtk_tree_path_free(path);
             return TRUE;
         case GDK_Return:
         case GDK_KP_Enter:
@@ -1124,10 +1128,72 @@ tree_view_key_pressed_cb(GtkWidget *tree, GdkEventKey *event, gpointer user_data
                 gtk_tree_view_collapse_row(GTK_TREE_VIEW(tree), path);
             else
                 gtk_tree_view_expand_row(GTK_TREE_VIEW(tree), path, FALSE /* !open_all */);
+            gtk_tree_path_free(path);
 	    return TRUE;
     }
 
+    gtk_tree_path_free(path);
     return FALSE;
 }
 
+/* 
+ * This function can be called from gtk_tree_view_column_set_cell_data_func()
+ * the user data must be the colum number.
+ * Present floats with two decimals 
+ */
+void
+float_data_func (GtkTreeViewColumn *column _U_,
+                           GtkCellRenderer   *renderer,
+                           GtkTreeModel      *model,
+                           GtkTreeIter       *iter,
+                           gpointer           user_data)
+   {
+     gfloat  float_val;
+     gchar   buf[20];
+	 char *savelocale;
+
+	 /* the col to get data from is in userdata */
+	 gint float_col = GPOINTER_TO_INT(user_data);
+
+     gtk_tree_model_get(model, iter, float_col, &float_val, -1);
+
+	 /* save the current locale */
+	 savelocale = setlocale(LC_NUMERIC, NULL);
+	 /* switch to "C" locale to avoid problems with localized decimal separators
+	  * in g_snprintf("%f") functions
+	  */
+	 setlocale(LC_NUMERIC, "C");
+
+     g_snprintf(buf, sizeof(buf), "%.2f", float_val);
+	 /* restore previous locale setting */
+	 setlocale(LC_NUMERIC, savelocale);
+
+     g_object_set(renderer, "text", buf, NULL);
+   }
+/* 
+ * This function can be called from gtk_tree_view_column_set_cell_data_func()
+ * the user data must be the colum number.
+ * Present value as hexadecimal. 
+ */
+
+void
+present_as_hex_func (GtkTreeViewColumn *column _U_,
+                           GtkCellRenderer   *renderer,
+                           GtkTreeModel      *model,
+                           GtkTreeIter       *iter,
+                           gpointer           user_data)
+   {
+     guint  val;
+     gchar   buf[35];
+
+	 /* the col to get data from is in userdata */
+	 gint col = GPOINTER_TO_INT(user_data);
+
+     gtk_tree_model_get(model, iter, col, &val, -1);
+
+     g_snprintf(buf, sizeof(buf), "0x%02x", val);
+	 /* restore previous locale setting */
+
+     g_object_set(renderer, "text", buf, NULL);
+   }
 
