@@ -1,7 +1,7 @@
 /*
  * Copyright 2004, Irene Ruengeler <i.ruengeler [AT] fh-muenster.de>
  *
- * $Id: sctp_byte_graph_dlg.c 25673 2008-07-08 02:43:40Z wmeier $
+ * $Id: sctp_byte_graph_dlg.c 27800 2009-03-19 17:49:11Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -359,7 +359,7 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 	else
 		distance = 5;
 
-	g_snprintf(label_string, 15, "%d", 0);
+	g_snprintf(label_string, sizeof(label_string), "%d", 0);
 	memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
 	layout = gtk_widget_create_pango_layout(u_data->io->draw_area, label_string);
 	pango_layout_get_pixel_size(layout, &label_width, &label_height);
@@ -371,8 +371,8 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 
 	if (u_data->io->offset != 0)
 	{
-		g_snprintf(label_string, 15, "%u", u_data->io->x1_tmp_sec);
-		memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
+		g_snprintf(label_string, sizeof(label_string), "%u", u_data->io->x1_tmp_sec);
+		memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), sizeof(label_string));
 		pango_layout_set_text(layout, label_string, -1);
 		pango_layout_get_pixel_size(layout, &lwidth, NULL);
 
@@ -428,13 +428,13 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 		if (i >= u_data->io->min_x && i % 1000000 != 0)
 		{
 			length = 5;
-			g_snprintf(label_string, 15, "%d", i%1000000);
+			g_snprintf(label_string, sizeof(label_string), "%d", i%1000000);
 
 			if (j % w == 0)
 			{
 				length = 10;
 
-				memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
+				memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), sizeof(label_string));
 				pango_layout_set_text(layout, label_string, -1);
 				pango_layout_get_pixel_size(layout, &lwidth, NULL);
 				gdk_draw_layout(u_data->io->pixmap,
@@ -475,8 +475,8 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 			              (guint32)(LEFT_BORDER + u_data->io->offset + (i - u_data->io->min_x) * u_data->io->x_interval),
 			              u_data->io->pixmap_height - BOTTOM_BORDER + 10);
 
-			g_snprintf(label_string, 15, "%d", sec);
-			memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
+			g_snprintf(label_string, sizeof(label_string), "%d", sec);
+			memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), sizeof(label_string));
 			pango_layout_set_text(layout, label_string, -1);
 			pango_layout_get_pixel_size(layout, &lwidth, NULL);
 			gdk_draw_layout(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,
@@ -487,9 +487,9 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 		}
 	}
 
-	g_strlcpy(label_string, "sec", 15);
+	g_strlcpy(label_string, "sec", sizeof(label_string));
 
-	memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
+	memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), sizeof(label_string));
 	pango_layout_set_text(layout, label_string, -1);
 	pango_layout_get_pixel_size(layout, &lwidth, NULL);
 	gdk_draw_layout(u_data->io->pixmap,
@@ -543,13 +543,13 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 			if (i >= u_data->io->min_y)
 			{
 				length = 5;
-				g_snprintf(label_string, 15, "%d", i);
+				g_snprintf(label_string, sizeof(label_string), "%d", i);
 
 				if (i%distance == 0 || (distance <= 5 && u_data->io->y_interval > 10))
 				{
 					length = 10;
 
-					memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
+					memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), sizeof(label_string));
 					pango_layout_set_text(layout, label_string, -1);
 					pango_layout_get_pixel_size(layout, &lwidth, NULL);
 					gdk_draw_layout(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,
@@ -999,7 +999,7 @@ on_button_release (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_uda
 				}
 			}
 			cf_goto_frame(&cfile, tmptsn->frame_number);
-			g_snprintf(label_string, 30, "(%.6f, %u)", x_value, y_value);
+			g_snprintf(label_string, sizeof(label_string), "(%.6f, %u)", x_value, y_value);
 			label_set = TRUE;
 
 			gdk_draw_line(u_data->io->pixmap,text_color, (gint)(event->x-2), (gint)(event->y), (gint)(event->x+2), (gint)(event->y));
