@@ -1,7 +1,7 @@
 /* voip_calls_dlg.c
  * VoIP calls summary addition for Wireshark
  *
- * $Id: voip_calls_dlg.c 27696 2009-03-11 06:43:46Z etxrab $
+ * $Id: voip_calls_dlg.c 28063 2009-04-16 03:18:10Z gerald $
  *
  * Copyright 2004, Ericsson , Spain
  * By Francisco Alcoba <francisco.alcoba@ericsson.com>
@@ -276,8 +276,8 @@ voip_calls_on_filter                    (GtkButton       *button _U_,
 	const gchar *filter_prepend;
 	gboolean isFirst = TRUE;
 	GList* list;
-	guint filter_length = 0;
-	guint max_filter_length = 2048;
+	size_t filter_length = 0;
+	size_t max_filter_length = 2048;
 	sip_calls_info_t *tmp_sipinfo;
 	isup_calls_info_t *tmp_isupinfo;
 	h323_calls_info_t *tmp_h323info;
